@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 import unittest
-from bottle import SimpleTemplate, TemplateError, view, template, touni, tob
+from bottle import SimpleTemplate, TemplateError, view, template, touni, tob, html_quote
 import re
 import traceback
 
@@ -51,6 +51,9 @@ class TestSimpleTemplate(unittest.TestCase):
         self.assertRenders('<{{var}}>', '<1.0>', var=1.0)
         self.assertRenders('<{{var}}>', '<[1, 2]>', var=[1,2])
 
+    def test_htmlutils_quote(self):
+        self.assertEquals('"&lt;&#039;&#13;&#10;&#9;&quot;\\&gt;"', html_quote('<\'\r\n\t"\\>'));
+
     def test_escape(self):
         self.assertRenders('<{{var}}>', '<b>', var='b')
         self.assertRenders('<{{var}}>', '<&lt;&amp;&gt;>',var='<&>')
@@ -247,6 +250,12 @@ class TestSimpleTemplate(unittest.TestCase):
     def test_bug_no_whitespace_before_stmt(self):
         self.assertRenders('\n{{var}}', '\nx', var='x')
 
+    def test_bug_block_keywords_eat_prefixed_code(self):
+        ''' #595: Everything before an 'if' statement is removed, resulting in
+            SyntaxError. '''
+        tpl = "% m = 'x' if True else 'y'\n{{m}}"
+        self.assertRenders(tpl, 'x')
+
 
 class TestSTPLDir(unittest.TestCase):
     def fix_ident(self, string):
